@@ -1,15 +1,15 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2006-2010, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2006-2011, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Team konohaken@googlegroups.com
  * All rights reserved.
  *
  * You may choose one of the following two licenses when you use konoha.
- * See www.konohaware.org/license.html for further information.
+ * If you want to use the latter license, please contact us.
  *
- * (1) GNU Lesser General Public License 3.0 (with K_UNDER_LGPL)
- * (2) Konoha Software Foundation License 1.0
+ * (1) GNU General Public License 3.0 (with K_UNDER_GPL)
+ * (2) Konoha Non-Disclosure License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
@@ -25,44 +25,21 @@
  *
  ****************************************************************************/
 
-#define USE_B     1
-#include<konoha.h>
+#include<konoha1.h>
+#include<errno.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-int main(int argc, const char **argv)
+int main(int argc, const char *argv[])
 {
-#if defined(K_USING_BTRON)
-	char **args = knh_tcstoeucs(argc, argv);
-#else
-	const char** args = argv;
-#endif
-	konoha_t konoha = konoha_open(4096);
-	int n = konoha_parseopt(konoha, argc, args);
-	if(argc - n == 0) {
-		konoha_shell(konoha, NULL);
-	}
-	else if(knh_isTestMode()) {
-		konoha_runTest(konoha, argc - n, args + n);
-	}
-	else {
-		int isCompileOnly = knh_Context_isCompiling(konoha.ctx);
-		if(konoha_load(konoha, B(args[n]), isCompileOnly) != -1) {
-			if(!isCompileOnly) {
-				konoha_runMain(konoha, argc - n, args + n);
-				if(knh_isInteractiveMode()) {
-					konoha_shell(konoha, NULL);
-				}
-			}
-		}
-		else {
-			KNH_SYSLOG(konoha.ctx, LOG_CRIT, "FailedLaunchScript", "script='%s'", args[n]);
-		}
-	}
+	int ret;
+	konoha_ginit(argc, argv);
+	konoha_t konoha = konoha_open();
+	ret = konoha_main(konoha, argc, argv);
 	konoha_close(konoha);
-	return 0;
+	return ret;
 }
 
 #ifdef __cplusplus
