@@ -1,15 +1,15 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2006-2010, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2006-2011, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Team konohaken@googlegroups.com
  * All rights reserved.
  *
  * You may choose one of the following two licenses when you use konoha.
- * See www.konohaware.org/license.html for further information.
+ * If you want to use the latter license, please contact us.
  *
- * (1) GNU Lesser General Public License 3.0 (with K_UNDER_LGPL)
- * (2) Konoha Software Foundation License 1.0
+ * (1) GNU General Public License 3.0 (with K_UNDER_GPL)
+ * (2) Konoha Non-Disclosure License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
@@ -38,15 +38,15 @@ extern "C" {
 #ifdef K_USING_SEMANTICS
 
 static
-knh_class_t knh_addSpecializedType(Ctx *ctx, knh_class_t cid, knh_class_t bcid, knh_Semantics_t *u);
+kclass_t knh_addSpecializedType(CTX ctx, kclass_t cid, kclass_t bcid, kSemantics *u);
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(knh_Int_t*) new_IntX(Ctx *ctx, knh_class_t cid, knh_int_t value)
+kInt* new_Int_X(CTX ctx, kclass_t cid, kint_t value)
 {
-	knh_Semantics_t *u = knh_getSemantics(ctx, cid);
+	kSemantics *u = knh_getSemantics(ctx, cid);
 	if(DP(u)->fichk(u, value)) {
-		knh_Int_t *n = (knh_Int_t*)new_H(ctx, FLAG_Int, CLASS_Int, cid);
+		kInt *n = (kInt*)new_H(ctx, FLAG_Int, CLASS_Int, cid);
 		n->n.ivalue = value;
 		return n;
 	}
@@ -63,12 +63,12 @@ KNHAPI(knh_Int_t*) new_IntX(Ctx *ctx, knh_class_t cid, knh_int_t value)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(knh_Float_t*) new_FloatX(Ctx *ctx, knh_class_t cid, knh_float_t value)
+kFloat* new_Float_X(CTX ctx, kclass_t cid, kfloat_t value)
 {
 	DBG_ASSERT_cid(cid);
-	knh_Semantics_t *u = knh_getSemantics(ctx, cid);
+	kSemantics *u = knh_getSemantics(ctx, cid);
 	if(DP(u)->ffchk(u, value)) {
-		knh_Float_t *f = (knh_Float_t*)new_H(ctx, FLAG_Float, CLASS_Float, cid);
+		kFloat *f = (kFloat*)new_H(ctx, FLAG_Float, CLASS_Float, cid);
 		f->n.fvalue = value;
 		return f;
 	}
@@ -78,38 +78,38 @@ KNHAPI(knh_Float_t*) new_FloatX(Ctx *ctx, knh_class_t cid, knh_float_t value)
 	}
 }
 
-/* ======================================================================== */
+/* ------------------------------------------------------------------------ */
 /* [Int] */
 
-static int knh_fichk__range(knh_Semantics_t *u, knh_int_t v)
+static int knh_fichk__range(kSemantics *u, kint_t v)
 {
 	return (DP(u)->imin <= v && v <= DP(u)->imax);
 }
 
-static int knh_fichk__umax(knh_Semantics_t *u, knh_uint_t v)
+static int knh_fichk__umax(kSemantics *u, kuint_t v)
 {
 	return (v <= DP(u)->umax);
 }
 
-static int knh_fichk__urange(knh_Semantics_t *u, knh_uint_t v)
+static int knh_fichk__urange(kSemantics *u, kuint_t v)
 {
 	return (DP(u)->umin <= v && v <= DP(u)->umax);
 }
 
-static int knh_ficmp__unsigned(knh_Semantics_t *u, knh_uint_t v1, knh_uint_t v2)
+static int knh_ficmp__unsigned(kSemantics *u, kuint_t v1, kuint_t v2)
 {
 	if(v1 == v2) return 0;
 	return (v1 > v2) ? 1 : -1;
 }
 
-static int knh_ffchk__range(knh_Semantics_t *u, knh_float_t v)
+static int knh_ffchk__range(kSemantics *u, kfloat_t v)
 {
 	return (DP(u)->fmin <= v && v <= DP(u)->fmax);
 }
 
-static int knh_ffcmp__step(knh_Semantics_t *u, knh_float_t v1, knh_float_t v2)
+static int knh_ffcmp__step(kSemantics *u, kfloat_t v1, kfloat_t v2)
 {
-	knh_float_t v = v1 - v2;
+	kfloat_t v = v1 - v2;
 	if(v >= DP(u)->fstep) return 1;
 	if(v <= -(DP(u)->fstep)) return -1;
 	return 0;
@@ -117,13 +117,13 @@ static int knh_ffcmp__step(knh_Semantics_t *u, knh_float_t v1, knh_float_t v2)
 
 /* ------------------------------------------------------------------------ */
 
-static void knh_Semantics_initIntRange(Ctx *ctx, knh_Semantics_t *u, knh_int_t min, knh_int_t max)
+static void knh_Semantics_initIntRange(CTX ctx, kSemantics *u, kint_t min, kint_t max)
 {
 	DP(u)->imin = min;
-	DP(u)->fmin = (knh_float_t)min;
+	DP(u)->fmin = (kfloat_t)min;
 	DP(u)->imax = max;
-	DP(u)->fmax = (knh_float_t)max;
-	DP(u)->fstep = K_FLOAT_ONE;
+	DP(u)->fmax = (kfloat_t)max;
+	DP(u)->fstep = KFLOAT_ONE;
 	if(min >= 0) {
 		DP(u)->ficmp = (knh_Ficmp)knh_ficmp__unsigned;
 		if(min == 0) {
@@ -135,7 +135,7 @@ static void knh_Semantics_initIntRange(Ctx *ctx, knh_Semantics_t *u, knh_int_t m
 		DP(u)->ffchk = knh_ffchk__range;
 	}
 	else {
-		if(min != K_INT_MIN || max != K_INT_MAX) {
+		if(min != KINT_MIN || max != KINT_MAX) {
 			DP(u)->fichk = knh_fichk__range;
 			DP(u)->ffchk = knh_ffchk__range;
 		}
@@ -145,16 +145,16 @@ static void knh_Semantics_initIntRange(Ctx *ctx, knh_Semantics_t *u, knh_int_t m
 
 /* ------------------------------------------------------------------------ */
 
-void knh_write_intx(Ctx *ctx, knh_OutputStream_t *w, knh_Semantics_t *u, knh_int_t v)
+void knh_write_intx(CTX ctx, kOutputStream *w, kSemantics *u, kint_t v)
 {
-	char buf[K_INT_FMTSIZ];
-	char *fmt = K_INT_FMT;
+	char buf[KINT_FMTSIZ];
+	char *fmt = KINT_FMT;
 	if(DP(u)->imin >= 0) {
-		fmt = K_UINT_FMT;
+		fmt = KUINT_FMT;
 	}
 	knh_snprintf(buf, sizeof(buf), fmt, v);
 	knh_write(ctx, w, B(buf));
-	knh_bytes_t tag = S_tobytes(DP(u)->tag);
+	kbytes_t tag = S_tobytes(DP(u)->tag);
 	if(tag.len > 0) {
 		knh_putc(ctx, w, '[');
 		knh_write(ctx, w, tag);
@@ -165,15 +165,15 @@ void knh_write_intx(Ctx *ctx, knh_OutputStream_t *w, knh_Semantics_t *u, knh_int
 /* ------------------------------------------------------------------------ */
 
 static
-void knh_Semantics_initFloatRange(Ctx *ctx, knh_Semantics_t *u, knh_float_t min, knh_float_t max, knh_float_t step)
+void knh_Semantics_initFloatRange(CTX ctx, kSemantics *u, kfloat_t min, kfloat_t max, kfloat_t step)
 {
 	DP(u)->fmin = min;
 	DP(u)->fmax = max;
 	DP(u)->fstep = step;
-	DP(u)->imax = (knh_int_t)max;
-	DP(u)->imin = (knh_int_t)min;
+	DP(u)->imax = (kint_t)max;
+	DP(u)->imin = (kint_t)min;
 
-	if(min >= K_FLOAT_ZERO) {
+	if(min >= KFLOAT_ZERO) {
 		DP(u)->ficmp = (knh_Ficmp)knh_ficmp__unsigned;
 		if(min == 0) {
 			DP(u)->fichk = (knh_Fichk)knh_fichk__umax;
@@ -184,39 +184,39 @@ void knh_Semantics_initFloatRange(Ctx *ctx, knh_Semantics_t *u, knh_float_t min,
 		DP(u)->ffchk = knh_ffchk__range;
 	}
 	else {
-		if(min != K_FLOAT_MIN || max != K_FLOAT_MAX) {
+		if(min != KFLOAT_MIN || max != KFLOAT_MAX) {
 			DP(u)->fichk = knh_fichk__range;
 			DP(u)->ffchk = knh_ffchk__range;
 		}
 	}
 
-	if(step != K_FLOAT_ZERO) {
+	if(step != KFLOAT_ZERO) {
 		DP(u)->ffcmp = knh_ffcmp__step;
 	}
 }
 
 /* ------------------------------------------------------------------------ */
 
-void knh_write_floatx(Ctx *ctx, knh_OutputStream_t *w, knh_Semantics_t *u, knh_float_t v)
+void knh_write_floatx(CTX ctx, kOutputStream *w, kSemantics *u, kfloat_t v)
 {
-	char *FMT = K_FLOAT_FMT;
+	char *FMT = KFLOAT_FMT;
 #if !defined(K_USING_NOFLOT)
-	knh_float_t step = DP(u)->fstep;
+	kfloat_t step = DP(u)->fstep;
 	if(0.1 <= step && step < 1.0) {
-		FMT = K_FLOAT_FMT1;
+		FMT = KFLOAT_FMT1;
 	}
 	else if(0.01 <= step && step < 0.1) {
-		FMT = K_FLOAT_FMT2;
+		FMT = KFLOAT_FMT2;
 	}
 	else if(0.001 <= step && step < 0.01) {
-		FMT = K_FLOAT_FMT3;
+		FMT = KFLOAT_FMT3;
 	}
 	else if(0.0001 <= step && step < 0.001) {
-		FMT = K_FLOAT_FMT4;
+		FMT = KFLOAT_FMT4;
 	}
 #endif
 	knh_write_ffmt(ctx, w, FMT, v);
-	knh_bytes_t tag = S_tobytes(DP(u)->tag);
+	kbytes_t tag = S_tobytes(DP(u)->tag);
 	if(tag.len > 0) {
 		knh_putc(ctx, w, '[');
 		knh_write(ctx, w, tag);
@@ -226,24 +226,24 @@ void knh_write_floatx(Ctx *ctx, knh_OutputStream_t *w, knh_Semantics_t *u, knh_f
 
 /* ------------------------------------------------------------------------ */
 
-static TCAST knh_IntX_FloatX(Ctx *ctx, knh_sfp_t *sfp, long rix)
+static TYPEMAP knh_IntX_FloatX(CTX ctx, ksfp_t *sfp _RIX)
 {
-	knh_float_t v = (knh_float_t)sfp[0].ivalue;
+	kfloat_t v = (kfloat_t)sfp[0].ivalue;
 	KNH_MAPPED_Float(ctx, sfp, v);
 }
 
 /* ------------------------------------------------------------------------ */
 
-static TCAST knh_FloatX_IntX(Ctx *ctx, knh_sfp_t *sfp, long rix)
+static TYPEMAP knh_FloatX_IntX(CTX ctx, ksfp_t *sfp _RIX)
 {
-	knh_int_t v = (knh_int_t)sfp[0].fvalue;
+	kint_t v = (kint_t)sfp[0].fvalue;
 	KNH_MAPPED_Int(ctx, sfp, v);
 }
 
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_Semantics_isVocab(knh_Semantics_t *u)
+int Semantics_isVocab(kSemantics *u)
 {
 	return IS_DictIdx(DP(u)->vocabDictIdx);
 }
@@ -251,7 +251,7 @@ int knh_Semantics_isVocab(knh_Semantics_t *u)
 /* ------------------------------------------------------------------------ */
 
 static
-knh_String_t *knh_Semantics_getVocabAt(Ctx *ctx, knh_Semantics_t *u, size_t n)
+kString *knh_Semantics_getVocabAt(CTX ctx, kSemantics *u, size_t n)
 {
 	return knh_DictIdx_get__fast(DP(u)->vocabDictIdx, n - DP(u)->imin);
 }
@@ -259,68 +259,68 @@ knh_String_t *knh_Semantics_getVocabAt(Ctx *ctx, knh_Semantics_t *u, size_t n)
 /* ------------------------------------------------------------------------ */
 
 static
-knh_int_t knh_Semantics_getVocabIdx(Ctx *ctx, knh_Semantics_t *u, knh_String_t *s)
+kint_t knh_Semantics_getVocabIdx(CTX ctx, kSemantics *u, kString *s)
 {
 	return knh_DictIdx_index(ctx, DP(u)->vocabDictIdx, S_tobytes(s)) + DP(u)->imin;
 }
 
 /* ------------------------------------------------------------------------ */
 
-static TCAST knh_IntX_Vocab(Ctx *ctx, knh_sfp_t *sfp, long rix)
+static TYPEMAP knh_IntX_Vocab(CTX ctx, ksfp_t *sfp _RIX)
 {
-	knh_Semantics_t *u = (knh_Semantics_t*)DP(sfp[1].mpr)->mapdata;
+	kSemantics *u = (kSemantics*)DP(sfp[1].mpr)->mapdata;
 	KNH_ASSERT(IS_Semantics(u));
 	RETURN_(knh_Semantics_getVocabAt(ctx, u, sfp[0].ivalue));
 }
 
 /* ------------------------------------------------------------------------ */
 
-static TCAST knh_Vocab_IntX(Ctx *ctx, knh_sfp_t *sfp, long rix)
+static TYPEMAP knh_Vocab_IntX(CTX ctx, ksfp_t *sfp _RIX)
 {
-	knh_Semantics_t *u = (knh_Semantics_t*)DP(sfp[1].mpr)->mapdata;
+	kSemantics *u = (kSemantics*)DP(sfp[1].mpr)->mapdata;
 	KNH_ASSERT(IS_Semantics(u));
 	KNH_MAPPED_Int(ctx, sfp, knh_Semantics_getVocabIdx(ctx, u, sfp[0].s));
 }
 
 /* ------------------------------------------------------------------------ */
 
-void knh_Semantics_reuse(Ctx *ctx, knh_Semantics_t *u, knh_class_t cid)
+void knh_Semantics_reuse(CTX ctx, kSemantics *u, kclass_t cid)
 {
-	knh_class_t bcid = ClassTBL(cid).bcid;
+	kclass_t bcid = ClassTBL(cid)->bcid;
 	if(bcid == CLASS_Int) {
-		knh_int_t v = 0;
+		kint_t v = 0;
 		if(!DP(u)->fichk(u, v)) {
 			v = DP(u)->imin;
 		}
 		KNH_ASSERT(DP(u)->ivalue == NULL);
-		KNH_INITv(DP(u)->ivalue, new_Int(ctx, cid, v));
+		KNH_INITv(DP(u)->ivalue, new_Int_(ctx, cid, v));
 		if(DP(u)->fvalue != NULL) {
-			knh_Translator_t *mpr = new_Translator(ctx, FLAG_Translator_Affine, DP(u)->ucid, cid, knh_FloatX_IntX, (Object*)u);
-			knh_addTranslator(ctx, mpr);
-			mpr = new_Translator(ctx, FLAG_Translator_Affine, cid, DP(u)->ucid, knh_IntX_FloatX, (Object*)u);
-			knh_addTranslator(ctx, mpr);
+			kTypeMap *mpr = new_TypeMap(ctx, FLAG_TypeMap_Affine, DP(u)->ucid, cid, knh_FloatX_IntX, (Object*)u);
+			knh_addTypeMap(ctx, mpr);
+			mpr = new_TypeMap(ctx, FLAG_TypeMap_Affine, cid, DP(u)->ucid, knh_IntX_FloatX, (Object*)u);
+			knh_addTypeMap(ctx, mpr);
 		}
 		if(DP(u)->svalue != NULL) {
-			if(knh_Semantics_isVocab(u)) {
-				knh_Translator_t *mpr = new_Translator(ctx, FLAG_Translator_Affine, cid, DP(u)->ucid, knh_IntX_Vocab, (Object*)u);
-				knh_addTranslator(ctx, mpr);
-				mpr = new_Translator(ctx, FLAG_Translator_Affine, DP(u)->ucid, cid, knh_Vocab_IntX, (Object*)u);
-				knh_addTranslator(ctx, mpr);
+			if(Semantics_isVocab(u)) {
+				kTypeMap *mpr = new_TypeMap(ctx, FLAG_TypeMap_Affine, cid, DP(u)->ucid, knh_IntX_Vocab, (Object*)u);
+				knh_addTypeMap(ctx, mpr);
+				mpr = new_TypeMap(ctx, FLAG_TypeMap_Affine, DP(u)->ucid, cid, knh_Vocab_IntX, (Object*)u);
+				knh_addTypeMap(ctx, mpr);
 			}
 		}
 	}
 	else if(bcid == CLASS_Float) {
-		knh_float_t v = 0;
+		kfloat_t v = 0;
 		if(!DP(u)->ffchk(u, v)) {
 			v = DP(u)->fmin;
 		}
 		KNH_ASSERT(DP(u)->fvalue == NULL);
-		KNH_INITv(DP(u)->fvalue, new_Float(ctx, cid, v));
+		KNH_INITv(DP(u)->fvalue, new_Float_(ctx, cid, v));
 		if(DP(u)->ivalue != NULL) {
-			knh_Translator_t *mpr = new_Translator(ctx, FLAG_Translator_Affine, cid, DP(u)->ucid, knh_FloatX_IntX, (Object*)u);
-			knh_addTranslator(ctx, mpr);
-			mpr = new_Translator(ctx, FLAG_Translator_Affine, DP(u)->ucid, cid, knh_IntX_FloatX, (Object*)u);
-			knh_addTranslator(ctx, mpr);
+			kTypeMap *mpr = new_TypeMap(ctx, FLAG_TypeMap_Affine, cid, DP(u)->ucid, knh_FloatX_IntX, (Object*)u);
+			knh_addTypeMap(ctx, mpr);
+			mpr = new_TypeMap(ctx, FLAG_TypeMap_Affine, DP(u)->ucid, cid, knh_IntX_FloatX, (Object*)u);
+			knh_addTypeMap(ctx, mpr);
 		}
 	}
 	else {
@@ -332,10 +332,10 @@ void knh_Semantics_reuse(Ctx *ctx, knh_Semantics_t *u, knh_class_t cid)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(knh_Semantics_t*) new_Enum(Ctx *ctx, char *tag, knh_bytes_t urn, knh_int_t min, knh_int_t max)
+kSemantics* new_Enum(CTX ctx, char *tag, kbytes_t urn, kint_t min, kint_t max)
 {
-	knh_class_t cid = new_ClassId(ctx);
-	knh_Semantics_t* u = (knh_Semantics_t*)new_Object_bcid(ctx, CLASS_Semantics, (int)cid);
+	kclass_t cid = new_ClassId(ctx);
+	kSemantics* u = (kSemantics*)new_Object_bcid(ctx, CLASS_Semantics, (int)cid);
 	DP(u)->ubcid = CLASS_Int;
 	KNH_SETv(ctx, DP(u)->urn, new_S(ctx, urn, NULL));
 	if(tag != NULL || tag[0] != 0) {
@@ -343,10 +343,10 @@ KNHAPI(knh_Semantics_t*) new_Enum(Ctx *ctx, char *tag, knh_bytes_t urn, knh_int_
 	}
 	knh_Semantics_initIntRange(ctx, u, min, max);
 	if(!DP(u)->fichk(u, 0)) {
-		KNH_INITv(DP(u)->ivalue, new_Int(ctx, cid, min));
+		KNH_INITv(DP(u)->ivalue, new_Int_(ctx, cid, min));
 	}
 	else {
-		KNH_INITv(DP(u)->ivalue, new_Int(ctx, cid, 0));
+		KNH_INITv(DP(u)->ivalue, new_Int_(ctx, cid, 0));
 	}
 	knh_addSpecializedType(ctx, cid, CLASS_Int, u);
 	return u;
@@ -355,10 +355,10 @@ KNHAPI(knh_Semantics_t*) new_Enum(Ctx *ctx, char *tag, knh_bytes_t urn, knh_int_
 /* ------------------------------------------------------------------------ */
 
 
-KNHAPI(knh_Semantics_t*) new_Unit(Ctx *ctx, char *tag, knh_bytes_t urn, knh_float_t min, knh_float_t max, knh_float_t step)
+kSemantics* new_Unit(CTX ctx, char *tag, kbytes_t urn, kfloat_t min, kfloat_t max, kfloat_t step)
 {
-	knh_class_t cid = new_ClassId(ctx);
-	knh_Semantics_t* u = (knh_Semantics_t*)new_Object_bcid(ctx, CLASS_Semantics, (int)cid);
+	kclass_t cid = new_ClassId(ctx);
+	kSemantics* u = (kSemantics*)new_Object_bcid(ctx, CLASS_Semantics, (int)cid);
 	DP(u)->ubcid = CLASS_Float;
 	KNH_SETv(ctx, DP(u)->urn, new_S(ctx, urn, NULL));
 	if(tag != NULL || tag[0] != 0) {
@@ -368,26 +368,26 @@ KNHAPI(knh_Semantics_t*) new_Unit(Ctx *ctx, char *tag, knh_bytes_t urn, knh_floa
 	knh_Semantics_initFloatRange(ctx, u, min, max, step);
 #if !defined(K_USING_NOFLOT)
 	if(!DP(u)->ffchk(u, 0.0)) {
-		KNH_INITv(DP(u)->fvalue, new_Float(ctx, cid, min));
+		KNH_INITv(DP(u)->fvalue, new_Float_(ctx, cid, min));
 	}
 	else {
-		KNH_INITv(DP(u)->fvalue, new_Float(ctx, cid, 0.0));
+		KNH_INITv(DP(u)->fvalue, new_Float_(ctx, cid, 0.0));
 	}
 #endif
 	knh_addSpecializedType(ctx, cid, CLASS_Float, u);
 	return u;
 }
 
-/* ======================================================================== */
+/* ------------------------------------------------------------------------ */
 /* [String] */
 
 static
-knh_String_t *knh_fsnew__dict(Ctx *ctx, knh_class_t cid, knh_bytes_t t, knh_String_t *memoNULL, int *foundError)
+kString *knh_fsnew__dict(CTX ctx, kclass_t cid, kbytes_t t, kString *memoNULL, int *foundError)
 {
-	knh_Semantics_t *u = knh_getSemantics(ctx, cid);
-	knh_index_t n = knh_DictIdx_index(ctx, DP(u)->vocabDictIdx, t);
+	kSemantics *u = knh_getSemantics(ctx, cid);
+	kindex_t n = knh_DictIdx_index(ctx, DP(u)->vocabDictIdx, t);
 	if(n == -1) {
-		knh_bytes_t tag;
+		kbytes_t tag;
 		if(knh_bytes_splitTag(t, &tag, &t)) {
 			n = knh_DictIdx_index(ctx, DP(u)->vocabDictIdx, t);
 		}
@@ -402,7 +402,7 @@ knh_String_t *knh_fsnew__dict(Ctx *ctx, knh_class_t cid, knh_bytes_t t, knh_Stri
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_fscmp__dict(knh_Semantics_t *u, knh_bytes_t v1, knh_bytes_t v2)
+int knh_fscmp__dict(kSemantics *u, kbytes_t v1, kbytes_t v2)
 {
 	return knh_DictIdx_index(NULL, DP(u)->vocabDictIdx, v1) - knh_DictIdx_index(NULL, DP(u)->vocabDictIdx, v2);
 }
@@ -410,30 +410,30 @@ int knh_fscmp__dict(knh_Semantics_t *u, knh_bytes_t v1, knh_bytes_t v2)
 /* ------------------------------------------------------------------------ */
 
 static
-knh_DictIdx_t* new_DictIdx__Array(Ctx *ctx, knh_Array_t *a)
+knh_DictIdx_t* new_DictIdx__Array(CTX ctx, kArray *a)
 {
 	knh_DictIdx_t *o = (knh_DictIdx_t*)new_H(ctx, FLAG_DictIdx, CLASS_DictIdx, CLASS_DictIdx);
 	KNH_INITv(o->terms, a);
 	KNH_INITv(o->termsDictSet, new_DictSet(ctx, knh_Array_size(a)));
 	o->offset = 0;
 	{
-		knh_uintptr_t i = 0;
+		kuintptr_t i = 0;
 		for(i = 0; i < knh_Array_size(a); i++) {
-			knh_String_t *s = (knh_String_t*)knh_Array_n(a, i);
+			kString *s = (kString*)knh_Array_n(a, i);
 			knh_DictSet_append(ctx, o->termsDictSet, s, i+1);
 		}
 	}
-	//knh_DictSet_toIgnoreCase(o->termsDictSet);
+	//kDictSetoIgnoreCase(o->termsDictSet);
 	return o;
 }
 
 /* ------------------------------------------------------------------------ */
 
 static
-TCAST knh_Ftranslator_vocabidx(Ctx *ctx, knh_sfp_t *sfp, long rix)
+TYPEMAP knh_Ftypemap_vocabidx(CTX ctx, ksfp_t *sfp _RIX)
 {
-	knh_Semantics_t *u = knh_getSemantics(ctx, knh_Object_cid(sfp[0].o));
-	knh_int_t n = knh_Semantics_getVocabIdx(ctx, u, sfp[0].s);
+	kSemantics *u = knh_getSemantics(ctx, O_cid(sfp[0].o));
+	kint_t n = knh_Semantics_getVocabIdx(ctx, u, sfp[0].s);
 	DBG_P("n = %d", (int)n);
 	KNH_MAPPED_Int(ctx, sfp, n);
 }
@@ -441,12 +441,12 @@ TCAST knh_Ftranslator_vocabidx(Ctx *ctx, knh_sfp_t *sfp, long rix)
 /* ------------------------------------------------------------------------ */
 
 static
-TCAST knh_Ftranslator_vocab(Ctx *ctx, knh_sfp_t *sfp, long rix)
+TYPEMAP knh_Ftypemap_vocab(CTX ctx, ksfp_t *sfp _RIX)
 {
-	knh_Semantics_t *u = (knh_Semantics_t*)DP(sfp[1].mpr)->mapdata;
+	kSemantics *u = (kSemantics*)DP(sfp[1].mpr)->mapdata;
 	size_t n = (size_t)(sfp[0].ivalue - DP(u)->imin);
 	DBG_P("n = %zd", n);
-	knh_Array_t *a = (DP(u)->vocabDictIdx)->terms;
+	kArray *a = (DP(u)->vocabDictIdx)->terms;
 	Object *s = KNH_NULL;
 	if(n < knh_Array_size(a)) {
 		s = knh_Array_n(a, n);
@@ -456,10 +456,10 @@ TCAST knh_Ftranslator_vocab(Ctx *ctx, knh_sfp_t *sfp, long rix)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(knh_Semantics_t*) new_Vocab(Ctx *ctx, char *tag, knh_bytes_t urn, int base, char **terms)
+kSemantics* new_Vocab(CTX ctx, char *tag, kbytes_t urn, int base, char **terms)
 {
-	knh_class_t cid = new_ClassId(ctx);
-	knh_Semantics_t* u = (knh_Semantics_t*)new_Object_bcid(ctx, CLASS_Semantics, (int)cid);
+	kclass_t cid = new_ClassId(ctx);
+	kSemantics* u = (kSemantics*)new_Object_bcid(ctx, CLASS_Semantics, (int)cid);
 	DP(u)->ubcid = CLASS_String;
 	KNH_SETv(ctx, DP(u)->urn, new_S(ctx, urn, NULL));
 	if(tag != NULL || tag[0] != 0) {
@@ -468,24 +468,24 @@ KNHAPI(knh_Semantics_t*) new_Vocab(Ctx *ctx, char *tag, knh_bytes_t urn, int bas
 	DP(u)->fsnew = knh_fsnew__dict;
 	DP(u)->fscmp = knh_fscmp__dict;
 	{
-		knh_Array_t *a = new_Array0(ctx, 0);
+		kArray *a = new_Array0(ctx, 0);
 		while(*terms != NULL) {
-			knh_String_t *s = new_T(*terms);
+			kString *s = new_T(*terms);
 			knh_Array_add_(ctx, a, UPCAST(s));
 			s->h.cid = cid;
 			terms++;
 		}
 		KNH_SETv(ctx, DP(u)->vocabDictIdx, new_DictIdx__Array(ctx, a));
-		knh_Semantics_initIntRange(ctx, u, (knh_int_t)base, (knh_int_t)(base + knh_Array_size(a) - 1));
+		knh_Semantics_initIntRange(ctx, u, (kint_t)base, (kint_t)(base + knh_Array_size(a) - 1));
 		KNH_ASSERT(knh_Array_size(a) > 0);
 		KNH_INITv(DP(u)->svalue, knh_Array_n(a, 0));
 	}
 	knh_addSpecializedType(ctx, cid, CLASS_String, u);
 	{
-		knh_Translator_t *mpr = new_Translator(ctx, FLAG_Translator_Total|FLAG_Translator_Const|FLAG_Translator_Final, cid, CLASS_Int, knh_Ftranslator_vocabidx, (Object*)u);
-		knh_addTranslator(ctx, mpr);
-		mpr = new_Translator(ctx, FLAG_Translator_Const|FLAG_Translator_Final, CLASS_Int, cid, knh_Ftranslator_vocab, (Object*)u);
-		knh_addTranslator(ctx, mpr);
+		kTypeMap *mpr = new_TypeMap(ctx, FLAG_TypeMap_Total|FLAG_TypeMap_Const|FLAG_TypeMap_Final, cid, CLASS_Int, knh_Ftypemap_vocabidx, (Object*)u);
+		knh_addTypeMap(ctx, mpr);
+		mpr = new_TypeMap(ctx, FLAG_TypeMap_Const|FLAG_TypeMap_Final, CLASS_Int, cid, knh_Ftypemap_vocab, (Object*)u);
+		knh_addTypeMap(ctx, mpr);
 	}
 	return u;
 }
@@ -493,10 +493,10 @@ KNHAPI(knh_Semantics_t*) new_Vocab(Ctx *ctx, char *tag, knh_bytes_t urn, int bas
 ///* ------------------------------------------------------------------------ */
 //
 //static
-//TCAST knh_Translator__fdict(Ctx *ctx, knh_sfp_t *sfp, long rix)
+//TYPEMAP knh_TypeMap__fdict(CTX ctx, ksfp_t *sfp _RIX)
 //{
 //	String *s = sfp[0].s;
-//	Semantics *u = knh_getSemantics(ctx, knh_Object_cid(sfp[0].o)].cspec;
+//	Semantics *u = knh_getSemantics(ctx, O_cid(sfp[0].o)].cspec;
 //	KNH_ASSERT(IS_Semantics(u));
 //	KNH_ASSERT(IS_DictIdx(DP(u)->vocabDictIdx));
 //
@@ -511,10 +511,10 @@ KNHAPI(knh_Semantics_t*) new_Vocab(Ctx *ctx, char *tag, knh_bytes_t urn, int bas
 //
 ///* ------------------------------------------------------------------------ */
 //
-//KNHAPI(void) knh_addVocabularyTranslator(Ctx *ctx, knh_class_t scid, char *text)
+//void knh_addVocabularyTypeMap(CTX ctx, kclass_t scid, char *text)
 //{
 //	DBG_ASSERT_cid(scid);
-//	knh_class_t tcid = knh_findcid(ctx, B(text));
+//	kclass_t tcid = knh_findcid(ctx, B(text));
 //
 //	if(tcid != CLASS_unknown || ctx->share->ClassTBL[tcid].bcid != tcid) {
 ////		Semantics *u = knh_getSemantics(ctx, scid].cspec;
@@ -531,16 +531,16 @@ KNHAPI(knh_Semantics_t*) new_Vocab(Ctx *ctx, char *tag, knh_bytes_t urn, int bas
 ////			TODO();
 ////			return;
 ////		}
-//		Translator *mpr = new_Translator(ctx, FLAG_Translator_Affine, scid, tcid,
-//				knh_Translator_fvocab, (Object*)KNH_NULL);
+//		TypeMap *mpr = new_TypeMap(ctx, FLAG_TypeMap_Affine, scid, tcid,
+//				knh_TypeMap_fvocab, (Object*)KNH_NULL);
 //		if(ctx->share->ClassTBL[scid].cmap == NULL) {
 //			knh_ClassTBL_t *TC = (knh_ClassTBL_t*)(&ctx->share->ClassTBL[scid]);
 //			KNH_INITv(TC->cmap, new_ClassMap0(ctx, 4));
 //		}
 //		knh_ClassMap_add(ctx, ctx->share->ClassTBL[scid].cmap, mpr);
 //
-//		mpr = new_Translator(ctx, FLAG_Translator_Affine, tcid, scid,
-//				knh_Translator_fvocab, (Object*)KNH_NULL);
+//		mpr = new_TypeMap(ctx, FLAG_TypeMap_Affine, tcid, scid,
+//				knh_TypeMap_fvocab, (Object*)KNH_NULL);
 //		if(ctx->share->ClassTBL[tcid].cmap == NULL) {
 //			knh_ClassTBL_t *TC = (knh_ClassTBL_t*)(&ctx->share->ClassTBL[tcid]);
 //			KNH_INITv(TC->cmap, new_ClassMap0(ctx, 4));
@@ -549,12 +549,12 @@ KNHAPI(knh_Semantics_t*) new_Vocab(Ctx *ctx, char *tag, knh_bytes_t urn, int bas
 //	}
 //}
 
-/* ======================================================================== */
+/* ------------------------------------------------------------------------ */
 /* [urnalias] */
 
-knh_bytes_t knh_getURNAlias(Ctx *ctx, knh_bytes_t aurn)
+kbytes_t knh_getURNAlias(CTX ctx, kbytes_t aurn)
 {
-	knh_String_t *s = (knh_String_t*)knh_DictMap_get(ctx,  DP(ctx->sys)->URNAliasDictMap, aurn);
+	kString *s = (kString*)knh_DictMap_get(ctx,  ctx->share->URNAliasDictMap, aurn);
 	if(IS_NOTNULL(s)) {
 		return S_tobytes(s);
 	}
@@ -563,48 +563,48 @@ knh_bytes_t knh_getURNAlias(Ctx *ctx, knh_bytes_t aurn)
 
 ///* ------------------------------------------------------------------------ */
 //
-//void knh_setAliasURN(Ctx *ctx, String *alias, String *urn)
+//void knh_setAliasURN(CTX ctx, String *alias, String *urn)
 //{
-//	if(knh_Context_isVerbose(ctx)) {
-//		String *s =(String*)knh_DictMap_get__b(ctx,  DP(ctx->sys)->URNAliasDictMap, S_tobytes(urn));
+//	if(CTX_isVerbose(ctx)) {
+//		String *s =(String*)knh_DictMap_get__b(ctx,  ctx->share->URNAliasDictMap, S_tobytes(urn));
 //		if(IS_NOTNULL(s)) {
-//			KNH_SYSLOG(ctx, LOG_WARNING, "Overriding %s %s", __tochar(alias), __tochar(s));
+//			KNH_SYSLOG(ctx, LOG_WARNING, "Overriding %s %s", __totext(alias), __totext(s));
 //		}
 //	}
-//	knh_DictMap_set(ctx, DP(ctx->sys)->URNAliasDictMap, alias, UPCAST(urn));
+//	knh_DictMap_set(ctx, ctx->share->URNAliasDictMap, alias, UPCAST(urn));
 //}
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_loadURNAliasData(Ctx *ctx, knh_StringData_t *data)
+void knh_loadScriptURNAliasData(CTX ctx, knh_StringData_t *data)
 {
-	knh_DictMap_t *map = DP(ctx->sys)->URNAliasDictMap;
+	kDictMap *mapptr = ctx->share->URNAliasDictMap;
 	knh_StringData_t *d = data;
 	while(d->name != NULL) {
-		knh_String_t *s =(knh_String_t*)knh_DictMap_get(ctx,  map, B(d->name));
+		kString *s =(kString*)knh_DictMap_get(ctx,  mapptr, B(d->name));
 		if(IS_NOTNULL(s) && !S_equals(s, B(d->value))) {
-			KNH_SYSLOG(ctx, LOG_WARNING, _("overriding alias %s %s as %s"), d->name, S_tochar(s), d->value);
+			KNH_SYSLOG(ctx, LOG_WARNING, _("overriding alias %s %s as %s"), d->name, S_totext(s), d->value);
 		}
 		d++;
 	}
 	d = data;
 	while(d->name != NULL) {
-		knh_String_t *n = new_T(d->name);
-		knh_DictMap_append(ctx, map, n, UPCAST(new_T(d->value)));
+		kString *n = new_T(d->name);
+		knh_DictMap_append(ctx, mapptr, n, UPCAST(new_T(d->value)));
 		d++;
 	}
 }
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_loadSemanticsFuncData(Ctx *ctx, knh_NamedPointerData_t *data)
+void knh_loadScriptSemanticsFuncData(CTX ctx, knh_FuncData_t *data)
 {
-	knh_DictSet_t *ds = DP(ctx->sys)->SpecFuncDictSet;
+	kDictSet *ds = ctx->share->SpecFuncDictSet;
 	OLD_LOCK(ctx, LOCK_SYSTBL, NULL);
 	while(data->name != NULL) {
 		DBG_P("adding.. '%s'", data->name);
-		knh_String_t *n = new_T(data->name);
-		knh_DictSet_set(ctx, ds, n, (knh_uintptr_t)data->ptr);
+		kString *n = new_T(data->name);
+		knh_DictSet_set(ctx, ds, n, (kuintptr_t)data->ptr);
 		data++;
 	}
 	OLD_UNLOCK(ctx, LOCK_SYSTBL, NULL);
@@ -613,14 +613,14 @@ KNHAPI(void) knh_loadSemanticsFuncData(Ctx *ctx, knh_NamedPointerData_t *data)
 /* ------------------------------------------------------------------------ */
 
 static
-knh_Semantics_t *new_SemanticsNULL(Ctx *ctx, knh_bytes_t urn)
+kSemantics *new_SemanticsNULL(CTX ctx, kbytes_t urn)
 {
-	knh_Semantics_t *u = NULL;
+	kSemantics *u = NULL;
 	OLD_LOCK(ctx, LOCK_SYSTBL, NULL);
-	knh_index_t loc = 0;
-	knh_bytes_t p = urn;
+	kindex_t loc = 0;
+	kbytes_t p = urn;
 	while(loc != -1) {
-		knh_fspec func = (knh_fspec)knh_DictSet_get(DP(ctx->sys)->SpecFuncDictSet, p);
+		knh_fspec func = (knh_fspec)knh_DictSet_get(ctx->share->SpecFuncDictSet, p);
 		if(func != NULL) {
 			u = func(ctx, urn);
 			if(u != NULL) {
@@ -638,12 +638,12 @@ L_UNLOCK:;
 	return u;
 }
 
-/* ======================================================================== */
+/* ------------------------------------------------------------------------ */
 /* [SPEC] */
 
-knh_Semantics_t *knh_getSemantics(Ctx *ctx, knh_class_t cid)
+kSemantics *knh_getSemantics(CTX ctx, kclass_t cid)
 {
-	knh_Semantics_t *u = (knh_Semantics_t*)ClassTBL(cid).cspec;
+	kSemantics *u = (kSemantics*)ClassTBL(cid)->cspec;
 	KNH_ASSERT(IS_Semantics(u));
 	return u;
 }
@@ -651,65 +651,65 @@ knh_Semantics_t *knh_getSemantics(Ctx *ctx, knh_class_t cid)
 /* ------------------------------------------------------------------------ */
 
 static
-Object *knh_ClassTBL_fdefault__ISPEC(Ctx *ctx, knh_class_t cid)
+Object *knh_ClassTBL_fdefault__ISPEC(CTX ctx, kclass_t cid)
 {
-	knh_Semantics_t *u = knh_getSemantics(ctx, cid);
+	kSemantics *u = knh_getSemantics(ctx, cid);
 	return UPCAST(DP(u)->ivalue);
 }
 
 /* ------------------------------------------------------------------------ */
 
 static
-Object *knh_ClassTBL_fdefault__FSPEC(Ctx *ctx, knh_class_t cid)
+Object *knh_ClassTBL_fdefault__FSPEC(CTX ctx, kclass_t cid)
 {
-	knh_Semantics_t *u = knh_getSemantics(ctx, cid);
+	kSemantics *u = knh_getSemantics(ctx, cid);
 	return UPCAST(DP(u)->fvalue);
 }
 
 /* ------------------------------------------------------------------------ */
 
 static
-Object *knh_ClassTBL_fdefault__SSPEC(Ctx *ctx, knh_class_t cid)
+Object *knh_ClassTBL_fdefault__SSPEC(CTX ctx, kclass_t cid)
 {
-	knh_Semantics_t *u = knh_getSemantics(ctx, cid);
+	kSemantics *u = knh_getSemantics(ctx, cid);
 	return UPCAST(DP(u)->svalue);
 }
 
 /* ------------------------------------------------------------------------ */
 
 static
-knh_class_t knh_addSpecializedType(Ctx *ctx, knh_class_t cid, knh_class_t supcid, knh_Semantics_t *u)
+kclass_t knh_addSpecializedType(CTX ctx, kclass_t cid, kclass_t supcid, kSemantics *u)
 {
-	knh_class_t bcid = knh_class_bcid(supcid);
+	kclass_t bcid = C_bcid(supcid);
 	char bufcn[CLASS__AME_BUFSIZ];
 	if(cid == CLASS_newid) {
 		cid = new_ClassId(ctx);
 	}
-	knh_snprintf(bufcn, sizeof(bufcn), KNH_SEMANTICS_FMT, CLASS__(bcid), S_tochar(DP(u)->urn));
+	knh_snprintf(bufcn, sizeof(bufcn), KNH_SEMANTICS_FMT, CLASS__(bcid), S_totext(DP(u)->urn));
 	knh_setClassName(ctx, cid, new_S(ctx, B(bufcn), NULL));
 	{
-		knh_ClassTBL_t *t = pClassTBL(ctx, cid);
+		knh_ClassTBL_t *t = varClassTBL(cid);
 		if((DP(u)->tag)->size > 0) {
-			knh_snprintf(bufcn, sizeof(bufcn), "%s:%s", CLASS__(bcid), S_tochar(DP(u)->tag));
+			knh_snprintf(bufcn, sizeof(bufcn), "%s:%s", CLASS__(bcid), S_totext(DP(u)->tag));
 			KNH_SETv(ctx, t->sname, new_S(ctx, B(bufcn), NULL));
 		}
-		DBG_P("added %d, '%s'", cid, S_tochar(t->lname));
+		DBG_P("added %d, '%s'", cid, S_totext(t->lname));
 
 		t->bcid   = bcid;
 		t->supcid = bcid;
-		t->cflag  = ClassTBL(bcid).cflag | FLAG_Class_Final;
-		t->oflag  = ClassTBL(bcid).oflag;
-		t->offset = ClassTBL(bcid).offset;
+		t->cflag  = ClassTBL(bcid)->cflag | FLAG_Class_Final;
+		t->oflag  = ClassTBL(bcid)->oflag;
+		t->offset = ClassTBL(bcid)->offset;
 
-		t->cspi  = ClassTBL(bcid).cspi;
-		t->size = ClassTBL(bcid).size;
-		t->bsize  = ClassTBL(bcid).bsize;
+		t->cspi  = ClassTBL(bcid)->cspi;
+		t->size = ClassTBL(bcid)->size;
+		t->bsize  = ClassTBL(bcid)->bsize;
 
 		DBG_ASSERT(t->fields == NULL);
 		KNH_INITv(t->methods, ClassTBL(supcid).methods);
 
 		if(t->cmap == NULL) {
-			KNH_INITv(t->tmaps, KNH_EMPTYLIST);
+			KNH_INITv(t->typemaps, K_EMPTYARRAY);
 		}
 		else {
 			KNH_ASSERT(IS_ClassMap(t->cmap));
@@ -741,14 +741,14 @@ knh_class_t knh_addSpecializedType(Ctx *ctx, knh_class_t cid, knh_class_t supcid
 /* ------------------------------------------------------------------------ */
 
 static
-knh_Semantics_t* knh_findSemanticsNULL(Ctx *ctx, knh_bytes_t lname)
+kSemantics* knh_findSemanticsNULL(CTX ctx, kbytes_t lname)
 {
-	knh_index_t loc = knh_bytes_index(lname, '{');
+	kindex_t loc = knh_bytes_index(lname, '{');
 	if(loc != -1) {
 		char *postfix = (char*)lname.buf + loc;
 		char cname[CLASS__AME_BUFSIZ];
 		knh_snprintf(cname, sizeof(cname), "Int%s", postfix);
-		knh_class_t cid = knh_getcid(ctx, B(cname));
+		kclass_t cid = knh_getcid(ctx, B(cname));
 		if(cid != CLASS_unknown) {
 			return knh_getSemantics(ctx, cid);
 		}
@@ -768,23 +768,23 @@ knh_Semantics_t* knh_findSemanticsNULL(Ctx *ctx, knh_bytes_t lname)
 
 /* ------------------------------------------------------------------------ */
 
-knh_class_t knh_findcidx(Ctx *ctx, knh_bytes_t lname)
+kclass_t knh_findcidx(CTX ctx, kbytes_t lname)
 {
-	knh_Semantics_t *u = 	knh_findSemanticsNULL(ctx, lname);
+	kSemantics *u = 	knh_findSemanticsNULL(ctx, lname);
 	if(u != NULL) {
-		knh_index_t loc = knh_bytes_index(lname, '{');
-		knh_class_t bcid = knh_getcid(ctx, knh_bytes_first(lname, loc));
+		kindex_t loc = knh_bytes_index(lname, '{');
+		kclass_t bcid = knh_getcid(ctx, knh_bytes_first(lname, loc));
 		return knh_addSpecializedType(ctx, CLASS_newid, bcid, u);
 	}
 	else {
-		knh_index_t loc = knh_bytes_index(lname, '{');
-		knh_bytes_t urn = knh_bytes_last(lname, loc+1); urn.len -= 1;
-		knh_class_t bcid = knh_getcid(ctx, knh_bytes_first(lname, loc));
+		kindex_t loc = knh_bytes_index(lname, '{');
+		kbytes_t urn = knh_bytes_last(lname, loc+1); urn.len -= 1;
+		kclass_t bcid = knh_getcid(ctx, knh_bytes_first(lname, loc));
 		DBG_P("cid=%d,%s", bcid, CLASS__(bcid));
 		u = new_SemanticsNULL(ctx, urn);
 		if(u != NULL) {
-			knh_class_t ucid = DP(u)->ucid;
-			knh_class_t ubcid = DP(u)->ubcid;
+			kclass_t ucid = DP(u)->ucid;
+			kclass_t ubcid = DP(u)->ubcid;
 			DBG_P("cid=%d,%s", ubcid, CLASS__(ubcid));
 			//knh_addSpecializedType(ctx, ucid, ubcid, u);
 			if(ubcid != bcid) {
